"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthBearer = void 0;
const joi_1 = __importDefault(require("joi"));
const apollo_server_core_1 = require("apollo-server-core");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const intercall_1 = require("../intercall");
const auth_type_1 = require("./auth-type");
const auth_type_verify_1 = require("./auth-type-verify");
const auth_id_token_1 = require("./auth-id-token");
class AuthBearer {
    constructor(urlOrfnVerify) {
        if ('string' === typeof urlOrfnVerify) {
            this.identityUrl = urlOrfnVerify;
        }
        else {
            this.fnVerify = urlOrfnVerify;
        }
        if (!this.identityUrl && !this.fnVerify) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'Identity URL is missing', { urlOrfnVerify });
        }
    }
    static init(urlOrfnVerify, serviceIdToken) {
        const self = new AuthBearer(urlOrfnVerify);
        // Retrieve service access info from the identity service
        const claim = auth_id_token_1.AuthIdToken.decode(serviceIdToken);
        self.filter = {
            orgId: claim.orgId,
            appDomain: claim.appDomain,
            productId: claim.productId
        };
        this.self = self;
        return self;
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'AuthBearer initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
    async verifyIdToken(authToken, flag) {
        if (!authToken?.info?.token) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Id token is missing');
        }
        const { rules, ...verifiedIdToken } = await this.callVerifyIdToken(authToken, flag);
        // Verification via internal function need to keep the rules to return to caller on the response message
        if (this.fnVerify) {
            authToken.info.authMetadata ??= {};
            authToken.info.authMetadata.rules = rules;
        }
        const ability = (0, auth_type_1.generateAbility)(rules);
        const idToken = {
            ...verifiedIdToken,
            ability
        };
        authToken.type = auth_type_1.AuthTokenType.Bearer;
        authToken.grant = true;
        authToken.info.auth = idToken;
        authToken.info.ability = ability;
        authToken.info.requestedBy = idToken.userId;
        authToken.info.requestedByName = idToken.userLogin;
        return authToken;
    }
    async callVerifyIdToken(authToken, flag) {
        // Decode token to get token details for validation
        if (!this.fnVerify || !flag) {
            const claim = auth_id_token_1.AuthIdToken.decode(authToken.info.token.replace('Bearer ', ''));
            if (!claim ||
                (claim.orgId && this.filter.orgId !== claim.orgId) ||
                (claim.appDomain && this.filter.appDomain !== claim.appDomain) ||
                (claim.productId && this.filter.productId !== claim.productId)) {
                (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Id token claim is out of scope', { claim, expected: this.filter });
            }
        }
        // Internal use by Identity service to verify idToken
        if (this.fnVerify) {
            // flag=true, identity service verifyIdToken endpoint
            const filter = flag ? authToken.info.authMetadata?.bearer || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.UNAUTHORISED, 'Id token header metadata is missing') : this.filter;
            return this.validateIdToken(await this.fnVerify(authToken.info.token, filter));
        }
        const query = (0, apollo_server_core_1.gql) `
            query VerifyIdToken {
                verifyIdToken {
                    orgId
                    appDomain
                    productId
                    userId
                    userLogin
                    localeCode
                    rules
                }
            }
        `;
        const headers = {
            authorization: authToken.info.token,
            'x-org-id': this.filter.orgId,
            'x-app-domain': this.filter.appDomain,
            'x-product-id': this.filter.productId
        };
        // Call Identity service to verify idToken, check blacklist, and return idToken info (e.g. rules, etc)
        return this.validateIdToken(await (0, intercall_1.callGraphQl)(this.identityUrl, query, 'verifyIdToken', headers));
    }
    validateIdToken(idToken) {
        const { error, value } = joi_1.default.object().keys(auth_type_verify_1.schemaVerifyIdToken).validate(idToken, { stripUnknown: true });
        if (error) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_FORMAT, error.message, {
                function: 'validateIdToken',
                details: error.details,
                text: { input: error.details?.[0]?.context?.peers || error.details?.[0]?.context?.key }
            });
        }
        return value;
    }
}
exports.AuthBearer = AuthBearer;
//# sourceMappingURL=auth-bearer.js.map