"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModelMiddleware = exports.OmitUpdateInput = exports.OmitCreateInput = exports.RecordStatus = void 0;
const async_hooks_1 = require("async_hooks");
const cm_libs_utils_1 = require("cm-libs-utils");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
var RecordStatus;
(function (RecordStatus) {
    RecordStatus["ACTIVE"] = "ACTIVE";
    RecordStatus["INACTIVE"] = "INACTIVE";
})(RecordStatus = exports.RecordStatus || (exports.RecordStatus = {}));
exports.OmitCreateInput = { createdBy: 0, createdByName: '', updatedAt: null };
exports.OmitUpdateInput = { updatedBy: 0, updatedByName: '' };
class BaseModelMiddleware {
    constructor(client, errorHandler) {
        client['$use'](async (params, next) => this.process(params, next));
        this.client = client;
        this.errorHandler = errorHandler;
        this.asyncLocalStorage = new async_hooks_1.AsyncLocalStorage();
    }
    async run(context, promise) {
        return this.asyncLocalStorage.run(context, async () => {
            return await promise();
        });
    }
    async process(params, next) {
        const start = (0, cm_libs_utils_1.timeNow)();
        switch (params.action) {
            case 'create':
            case 'createMany':
                params = this.packCreateArgs(params);
                break;
            case 'update':
            case 'updateMany':
                params = this.packUpdateArgs(params);
                break;
            case 'upsert':
                params = this.packUpsertArgs(params);
                break;
        }
        try {
            const result = await next(params);
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.NORMAL, () => `Run ${params.model}.${params.action} affected ${affectedCount(params.action, result)} record(s) took ${(0, cm_libs_utils_1.timeElapsed)(start)} ms`);
            return result;
        }
        catch (error) {
            const { accessInfo } = this.asyncLocalStorage.getStore();
            this.errorHandler ? this.errorHandler(error, params, accessInfo) : (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.DATABASE_ERROR, { params, accessInfo });
        }
    }
    packCreateArgs(params) {
        const { schemas, accessInfo } = this.asyncLocalStorage.getStore();
        const schema = schemas?.schemaCreateInput;
        const requestedBy = accessInfo.requestedBy;
        const requestedByName = accessInfo.requestedByName;
        if (!Array.isArray(params.args.data)) {
            params.args.data = {
                ...validatedValues(params.args.data, schema),
                createdBy: requestedBy,
                createdByName: requestedByName
            };
            return params;
        }
        for (let i = 0; params.args.data.length > i; i++) {
            params.args.data[i] = {
                ...validatedValues(params.args.data[i], schema),
                createdBy: requestedBy,
                createdByName: requestedByName
            };
        }
        return params;
    }
    packUpdateArgs(params) {
        const { schemas, accessInfo } = this.asyncLocalStorage.getStore();
        const schema = schemas?.schemaUpdateInput;
        const requestedBy = accessInfo.requestedBy;
        const requestedByName = accessInfo.requestedByName;
        if (!Array.isArray(params.args.data)) {
            params.args.data = {
                ...validatedValues(params.args.data, schema),
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
            return params;
        }
        for (let i = 0; params.args.data.length > i; i++) {
            params.args.data[i] = {
                ...validatedValues(params.args.data[i], schema),
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
        }
        return params;
    }
    packUpsertArgs(params) {
        const { schemas, accessInfo } = this.asyncLocalStorage.getStore();
        const schemaCreateInput = schemas?.schemaCreateInput;
        const schemaUpdateInput = schemas?.schemaUpdateInput;
        const requestedBy = accessInfo.requestedBy;
        const requestedByName = accessInfo.requestedByName;
        params.args.create = {
            ...validatedValues(params.args.create, schemaCreateInput),
            createdBy: requestedBy,
            createdByName: requestedByName
        };
        params.args.update = {
            ...validatedValues(params.args.update, schemaUpdateInput),
            updatedBy: requestedBy,
            updatedByName: requestedByName
        };
        return params;
    }
}
exports.BaseModelMiddleware = BaseModelMiddleware;
function affectedCount(action, result) {
    let count;
    switch (action) {
        case 'findUnique':
        case 'update':
            count = result ? 1 : 0;
            break;
        case 'findMany':
        case 'queryRaw':
            count = result?.length;
            break;
        case 'count':
        case 'executeRaw':
            count = Number(result);
            break;
        case 'create':
        case 'upsert':
            count = 1;
            break;
        case 'createMany':
        case 'updateMany':
            count = result?.count;
    }
    return count || 0;
}
function validatedValues(data, schema) {
    const { error, value } = schema ? schema.validate(data, { stripUnknown: true }) : { error: undefined, value: data };
    if (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.VALIDATION_ERROR, { data });
    }
    return value;
}
//# sourceMappingURL=model-middleware.js.map